@@ -12,7 +12,7 @@
 
 G_BEGIN_DECLS
 
-struct SoupURI {
+struct _SoupURI {
 	const char *scheme;
 
 	char       *user;
@@ -74,10 +74,12 @@ void      soup_uri_set_query_from_form   (SoupURI    *uri,
 					  GHashTable *form);
 void      soup_uri_set_query_from_fields (SoupURI    *uri,
 					  const char *first_field,
-					  ...);
+					  ...) G_GNUC_NULL_TERMINATED;
 void      soup_uri_set_fragment          (SoupURI    *uri,
 					  const char *fragment);
 
+#define   SOUP_URI_VALID_FOR_HTTP(uri) ((uri) && ((uri)->scheme == SOUP_URI_SCHEME_HTTP || (uri)->scheme == SOUP_URI_SCHEME_HTTPS) && (uri)->host)
+
 G_END_DECLS
 
 #endif /*SOUP_URI_H*/
