@@ -10,7 +10,7 @@
 #include "test-utils.h"
 
 typedef struct {
-	char *name, *value;
+	const char *name, *value;
 } Header;
 
 static struct RequestTest {
@@ -243,6 +243,17 @@ static struct RequestTest {
 	  }
 	},
 
+	/* qv bug 579318, do_bad_header_tests() below */
+	{ "Req w/ mangled header",
+	  "GET / HTTP/1.1\r\nHost: example.com\r\nFoo one\r\nBar: two\r\n", -1,
+	  SOUP_STATUS_OK,
+	  "GET", "/", SOUP_HTTP_1_1,
+	  { { "Host", "example.com" },
+	    { "Bar", "two" },
+	    { NULL }
+	  }
+	},
+
 	/************************/
 	/*** INVALID REQUESTS ***/
 	/************************/
@@ -303,13 +314,6 @@ static struct RequestTest {
 	  { { NULL } }
 	},
 
-	{ "Header line with no ':'",
-	  "GET / HTTP/1.1\r\nHost example.com\r\n", -1,
-	  SOUP_STATUS_BAD_REQUEST,
-	  NULL, NULL, -1,
-	  { { NULL } }
-	},
-
 	{ "No terminating CRLF",
 	  "GET / HTTP/1.1\r\nHost: example.com", -1,
 	  SOUP_STATUS_BAD_REQUEST,
@@ -317,27 +321,6 @@ static struct RequestTest {
 	  { { NULL } }
 	},
 
-	{ "Blank line before headers",
-	  "GET / HTTP/1.1\r\n\r\nHost: example.com\r\n", -1,
-	  SOUP_STATUS_BAD_REQUEST,
-	  NULL, NULL, -1,
-	  { { NULL } }
-	},
-
-	{ "Blank line in headers",
-	  "GET / HTTP/1.1\r\nHost: example.com\r\n\r\nConnection: close\r\n", -1,
-	  SOUP_STATUS_BAD_REQUEST,
-	  NULL, NULL, -1,
-	  { { NULL } }
-	},
-
-	{ "Blank line after headers",
-	  "GET / HTTP/1.1\r\nHost: example.com\r\nConnection: close\r\n\r\n", -1,
-	  SOUP_STATUS_BAD_REQUEST,
-	  NULL, NULL, -1,
-	  { { NULL } }
-	},
-
 	{ "Unrecognized expectation",
 	  "GET / HTTP/1.1\r\nHost: example.com\r\nExpect: the-impossible\r\n", -1,
 	  SOUP_STATUS_EXPECTATION_FAILED,
@@ -468,6 +451,7 @@ static struct ResponseTest {
 	  }
 	},
 
+	/* Shoutcast support */
 	{ "Shoutcast server not-quite-HTTP",
 	  "ICY 200 OK\r\nFoo: bar\r\n", -1,
 	  SOUP_HTTP_1_0, SOUP_STATUS_OK, "OK",
@@ -476,6 +460,16 @@ static struct ResponseTest {
 	  }
 	},
 
+	/* qv bug 579318, do_bad_header_tests() below */
+	{ "Response w/ mangled header",
+	  "HTTP/1.1 200 ok\r\nFoo: one\r\nBar two:2\r\nBaz: three\r\n", -1,
+	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "ok",
+	  { { "Foo", "one" },
+	    { "Baz", "three" },
+	    { NULL }
+	  }
+	},
+
 	/*************************/
 	/*** INVALID RESPONSES ***/
 	/*************************/
@@ -549,9 +543,9 @@ static struct ResponseTest {
 static const int num_resptests = G_N_ELEMENTS (resptests);
 
 static struct QValueTest {
-	char *header_value;
-	char *acceptable[7];
-	char *unacceptable[2];
+	const char *header_value;
+	const char *acceptable[7];
+	const char *unacceptable[2];
 } qvaluetests[] = {
 	{ "text/plain; q=0.5, text/html,\t  text/x-dvi; q=0.8, text/x-c",
 	  { "text/html", "text/x-c", "text/x-dvi", "text/plain", NULL },
@@ -599,7 +593,7 @@ check_headers (Header *headers, SoupMessageHeaders *hdrs)
 			ok = FALSE;
 			break;
 		}
-		value = soup_message_headers_get (hdrs, headers[i].name);
+		value = soup_message_headers_get_list (hdrs, headers[i].name);
 		if (strcmp (value, headers[i].value) != 0) {
 			ok = FALSE;
 			break;
@@ -839,7 +833,7 @@ do_rfc2231_tests (void)
 	soup_message_headers_set_content_disposition (hdrs, "attachment", params);
 	g_hash_table_destroy (params);
 
-	header = soup_message_headers_get (hdrs, "Content-Disposition");
+	header = soup_message_headers_get_one (hdrs, "Content-Disposition");
 	if (!strcmp (header, RFC2231_TEST_HEADER))
 		debug_printf (1, "  encoded OK\n");
 	else {
@@ -873,6 +867,157 @@ do_rfc2231_tests (void)
 
 	g_hash_table_destroy (params);
 	soup_message_headers_free (hdrs);
+
+	debug_printf (1, "\n");
+}
+
+#define CONTENT_TYPE_TEST_MIME_TYPE "text/plain"
+#define CONTENT_TYPE_TEST_ATTRIBUTE "charset"
+#define CONTENT_TYPE_TEST_VALUE     "US-ASCII"
+#define CONTENT_TYPE_TEST_HEADER    "text/plain; charset=\"US-ASCII\""
+
+#define CONTENT_TYPE_BAD_HEADER     "plain text, not text/html"
+
+static void
+do_content_type_tests (void)
+{
+	SoupMessageHeaders *hdrs;
+	GHashTable *params;
+	const char *header, *mime_type;
+
+	debug_printf (1, "Content-Type tests\n");
+
+	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
+	params = g_hash_table_new (g_str_hash, g_str_equal);
+	g_hash_table_insert (params, CONTENT_TYPE_TEST_ATTRIBUTE,
+			     CONTENT_TYPE_TEST_VALUE);
+	soup_message_headers_set_content_type (hdrs, CONTENT_TYPE_TEST_MIME_TYPE, params);
+	g_hash_table_destroy (params);
+
+	header = soup_message_headers_get_one (hdrs, "Content-Type");
+	if (!strcmp (header, CONTENT_TYPE_TEST_HEADER))
+		debug_printf (1, "  encoded OK\n");
+	else {
+		debug_printf (1, "  encoding FAILED!\n    expected: %s\n    got:      %s\n",
+			      CONTENT_TYPE_TEST_HEADER, header);
+		errors++;
+	}
+
+	soup_message_headers_clear (hdrs);
+	soup_message_headers_append (hdrs, "Content-Type",
+				     CONTENT_TYPE_TEST_MIME_TYPE);
+	/* Add a second Content-Type header: should be ignored */
+	soup_message_headers_append (hdrs, "Content-Type",
+				     CONTENT_TYPE_TEST_MIME_TYPE);
+
+	mime_type = soup_message_headers_get_content_type (hdrs, &params);
+	if (!mime_type) {
+		debug_printf (1, "  decoding FAILED!\n    could not parse\n");
+		errors++;
+	}
+
+	if (mime_type && strcmp (mime_type, CONTENT_TYPE_TEST_MIME_TYPE) != 0) {
+		debug_printf (1, "  decoding FAILED!\n    bad returned MIME type: %s\n",
+			      mime_type);
+		errors++;
+	} else if (params && g_hash_table_size (params) != 0) {
+		debug_printf (1, "  decoding FAILED!\n    params contained %d params (should be 0)\n",
+			      g_hash_table_size (params));
+		errors++;
+	} else
+		debug_printf (1, "  decoded OK\n");
+
+	if (params)
+		g_hash_table_destroy (params);
+
+	soup_message_headers_clear (hdrs);
+	soup_message_headers_append (hdrs, "Content-Type",
+				     CONTENT_TYPE_BAD_HEADER);
+	mime_type = soup_message_headers_get_content_type (hdrs, &params);
+	if (mime_type) {
+		debug_printf (1, "  Bad content rejection FAILED!\n");
+		errors++;
+	} else
+		debug_printf (1, "  Bad content rejection OK\n");
+
+	soup_message_headers_free (hdrs);
+
+	debug_printf (1, "\n");
+}
+
+struct {
+	const char *name, *value;
+} test_params[] = {
+	{ "one", "foo" },
+	{ "two", "test with spaces" },
+	{ "three", "test with \"quotes\" and \\s" },
+	{ "four", NULL },
+	{ "five", "test with \xC3\xA1\xC3\xA7\xC4\x89\xC3\xA8\xC3\xB1\xC5\xA3\xC5\xA1" }
+};
+
+#define TEST_PARAMS_RESULT "one=\"foo\", two=\"test with spaces\", three=\"test with \\\"quotes\\\" and \\\\s\", four, five*=UTF-8''test%20with%20%C3%A1%C3%A7%C4%89%C3%A8%C3%B1%C5%A3%C5%A1"
+
+static void
+do_append_param_tests (void)
+{
+	GString *params;
+	int i;
+
+	debug_printf (1, "soup_header_g_string_append_param() tests\n");
+
+	params = g_string_new (NULL);
+	for (i = 0; i < G_N_ELEMENTS (test_params); i++) {
+		if (i > 0)
+			g_string_append (params, ", ");
+		soup_header_g_string_append_param (params,
+						   test_params[i].name,
+						   test_params[i].value);
+	}
+	if (strcmp (params->str, TEST_PARAMS_RESULT) != 0) {
+		debug_printf (1, "  FAILED!\n    expected: %s\n    got: %s\n",
+			      TEST_PARAMS_RESULT, params->str);
+		errors++;
+	} else
+		debug_printf (1, "  OK\n");
+	g_string_free (params, TRUE);
+
+	debug_printf (1, "\n");
+}
+
+static const struct {
+	const char *description, *name, *value;
+} bad_headers[] = {
+	{ "Empty name", "", "value" },
+	{ "Name with spaces", "na me", "value" },
+	{ "Name with colon", "na:me", "value" },
+	{ "Name with CR", "na\rme", "value" },
+	{ "Name with LF", "na\nme", "value" },
+	{ "Name with tab", "na\tme", "value" },
+	{ "Value with CR", "name", "val\rue" },
+	{ "Value with LF", "name", "val\nue" },
+	{ "Value with LWS", "name", "val\r\n ue" }
+};
+
+static void
+do_bad_header_tests (void)
+{
+	SoupMessageHeaders *hdrs;
+	int i;
+
+	debug_printf (1, "bad header rejection tests\n");
+
+	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
+	for (i = 0; i < G_N_ELEMENTS (bad_headers); i++) {
+		debug_printf (1, "  %s\n", bad_headers[i].description);
+		expect_warning = TRUE;
+		soup_message_headers_append (hdrs, bad_headers[i].name,
+					     bad_headers[i].value);
+		if (expect_warning) {
+			expect_warning = FALSE;
+			debug_printf (1, "    FAILED: soup_message_headers_append() did not reject it\n");
+			errors++;
+		}
+	}
 }
 
 int
@@ -884,6 +1029,9 @@ main (int argc, char **argv)
 	do_response_tests ();
 	do_qvalue_tests ();
 	do_rfc2231_tests ();
+	do_content_type_tests ();
+	do_append_param_tests ();
+	do_bad_header_tests ();
 
 	test_cleanup ();
 	return errors != 0;
